@@ -229,7 +229,7 @@ static void setDeviceToCd(KCompactDisc *cd, struct cdrom_drive *drive)
 #if defined(HAVE_CDDA_IOCTL_DEVICE)
 	cd->setDevice(QLatin1String( drive->ioctl_device_name ), 50, false);
 #elif defined(__FreeBSD__) || defined(__DragonFly__)
-	// FreeBSD's cdparanoia as of january 5th 2006 has rather broken
+	// FreeBSD's cdparanoia as of January 5th 2006 has rather broken
 	// support for non-SCSI devices. Although it finds ATA cdroms just
 	// fine, there is no straightforward way to discover the device
 	// name associated with the device, which throws the rest of audiocd
@@ -451,6 +451,19 @@ bool AudioCDProtocol::getSectorsForRequest(struct cdrom_drive * drive, long & fi
 	return true;
 }
 
+static uint findInformationFileNumber(const QString &filename, uint max) {
+	if (filename == QString::fromLatin1("%1.txt").arg(i18n(CDDB_INFORMATION)))
+		return 1;
+
+	for (uint i = 2; i <= max; ++i) {
+		if (filename == QString::fromLatin1("%1_%2.txt").arg(i18n(CDDB_INFORMATION)).arg(i)) {
+			return i;
+		}
+	}
+
+	return max + 1;
+}
+
 void AudioCDProtocol::get(const QUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
@@ -460,10 +473,7 @@ void AudioCDProtocol::get(const QUrl & url)
 	}
 
 	if( d->fname.contains(i18n(CDDB_INFORMATION))){
-		uint choice = 1;
-		if(d->fname != QString::fromLatin1("%1.txt").arg(i18n(CDDB_INFORMATION))){
-			choice= d->fname.section(QLatin1Char( '_' ),1,1).section(QLatin1Char( '.' ),0,0).toInt();
-		}
+		const uint choice = findInformationFileNumber(d->fname, d->cddbList.count());
 		uint count = 1;
 		CDInfoList::iterator it;
 		bool found = false;
@@ -564,6 +574,39 @@ void AudioCDProtocol::stat(const QUrl & url)
 		return;
 	}
 
+	if (d->which_dir == Info) {
+		// This is the info dir or one of the info files
+		if (d->fname.isEmpty()) {
+			// The info dir
+			const mode_t _umask = ::umask(0);
+			::umask(_umask);
+			UDSEntry entry;
+			entry.INSERT(KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F")));
+			entry.INSERT(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+			entry.INSERT(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
+			entry.INSERT(KIO::UDSEntry::UDS_SIZE, d->cddbList.count());
+			statEntry(entry);
+			finished();
+			return;
+		} else if( d->fname.contains(i18n(CDDB_INFORMATION))) {
+			// choice is 1-indexed so we need <= and -1 when accessing d->cddbList
+			const uint choice = findInformationFileNumber(d->fname, d->cddbList.count());
+			if (choice <= (uint)d->cddbList.count()) {
+				// It's a valid info file
+				const mode_t _umask = ::umask(0);
+				::umask(_umask);
+				UDSEntry entry;
+				entry.INSERT(KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F")));
+				entry.INSERT(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+				entry.INSERT(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
+				entry.INSERT(KIO::UDSEntry::UDS_SIZE, d->cddbList.at(choice - 1).toString().toLatin1().size());
+				statEntry(entry);
+				finished();
+				return;
+			}
+		}
+	}
+
 	const bool isFile = !d->fname.isEmpty();
 
 	// the track number. 0 if ripping
@@ -630,7 +673,7 @@ static void app_file(UDSEntry& e, const QString & n, size_t s, const QString &mi
 	e.INSERT( KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
 	// Use current date and time to avoid confusions. See https://bugs.kde.org/show_bug.cgi?id=400114
 	e.INSERT( KIO::UDSEntry::UDS_MODIFICATION_TIME, QDateTime::currentDateTime().toTime_t ());
-	e.INSERT( KIO::UDSEntry::UDS_ACCESS, 0400);
+	e.INSERT( KIO::UDSEntry::UDS_ACCESS, 0644);
 	e.INSERT( KIO::UDSEntry::UDS_SIZE, s);
 	if (!mimetype.isEmpty())
 		e.INSERT( KIO::UDSEntry::UDS_MIME_TYPE, mimetype);
@@ -958,13 +1001,13 @@ void AudioCDProtocol::paranoiaRead(
 			diff = (diff*(unsigned long)((100/(float)end)*(end-cur)))/2;
 			// Need 1% of data calculated as initial buffer, use %2 to be safe
 			if( percentDone < .02 ){
-				//qDebug("val: %f, diff: %ld", ((float)cur/(float)end), diff);
+				//qCDebug(AUDIOCD_KIO_LOG) << "val: " << (float)cur/(float)end << " diff: " << diff;
 				diff = 0;
 			}
 
 			// We are growing larger, increase total.
 			if(lastSize < estSize){
-				//qDebug("lastGuess: %ld, guess: %ld diff: %ld", lastSize, estSize, diff);
+				//qCDebug(AUDIOCD_KIO_LOG) << "lastGuess: " << lastSize << ", guess: " << estSize << ", diff: " << diff;
 				totalSize(estSize+diff);
 				lastSize = estSize+diff;
 			}
@@ -975,7 +1018,7 @@ void AudioCDProtocol::paranoiaRead(
 					margin = 7;
 				unsigned long low = lastSize - lastSize/margin;
 				if(estSize < low){
-					//qDebug("low: %ld, estSize: %ld, num: %i", low, estSize, margin);
+					//qCDebug(AUDIOCD_KIO_LOG) << "low: " << low << ", estSize: " << estSize << ", num: " << margin;
 					totalSize( estSize );
 					lastSize = estSize;
 				}
@@ -984,7 +1027,7 @@ void AudioCDProtocol::paranoiaRead(
 		/**
 		 * End estimation.
 		 */
-		//qDebug("processed: %d, totalSize: %d", processed, estSize);
+		//qCDebug(AUDIOCD_KIO_LOG) << "processed: " << processed << ", totalSize: " << estSize;
 		processedSize(processed);
 	}
 
@@ -1090,9 +1133,9 @@ void AudioCDProtocol::loadSettings()
 		d->fileLocationTemplate = QString();
 	d->rsearch = groupFileName.readEntry("regexp_search");
 	d->rreplace = groupFileName.readEntry("regexp_replace");
-	// if the regular expressions are enclosed in qoutes. remove them
+	// if the regular expressions are enclosed in quotes. Remove them
 	// otherwise it is not possible to search for a space " ", since an empty (only spaces) value is not
-	// supported by KConfig, so the space has to be qouted, but then here the regexp searches really for " "
+	// supported by KConfig, so the space has to be quoted, but then here the regexp searches really for " "
 	// instead of just the space. Alex
 	QRegExp qoutedString( QLatin1String( "^\".*\"$" ));
 	if (qoutedString.exactMatch(d->rsearch))
